/**
 * 呼び出し元サイトに設置する固定スクリプト。
 * 文字コードは、UTF-8を指定します。
 * $Id$
 */

/**
 * 読み込み失敗と判断し、エラー処理を行うまでの時間。ミリ秒。
 */
var check_error_msec = 2000;

/**
 * 広告サーバードメイン
 */
var sannex_server_domain = 'shopping.sponichi.co.jp';


/**
 * 内部変数
 */
var banner_last_click;

var mall_name_list;

var check_banner_display;

var mall_cookie_valid_days;

/**
 * データの初期化。このファイルが読み込まれた直後に実行。
 * @return
 */
function init() {
	
	// バナーの表示履歴
	banner_last_click = new Array();
	
	// 利用可能モール一覧
	mall_name_list = new Array('YAHOO_SHOPPING', 'RAKUTEN_SHOPPING', 'AMAZON_SHOPPING', 'RAKUTEN_BOOKS', 'YAHOO_AUCTION', 'VALUECOMMERCE_PRODUCT', 'ORIGINAL_SHOPPING');
	
	// バナーが表示されたか？
	check_banner_display = new Array();
	
	// クッキー有効日数
	mall_cookie_valid_days = new Array();
	
	// 配列にshuffleを追加
	Array.prototype.shuffle = function() {
		var i = this.length;
		while(i){
			var j = Math.floor(Math.random() * i);
			var t = this[--i];
			this[i] = this[j];
			this[j] = t;
		}
		return this;
	}
	
}

/**
 * 呼び出し元から呼び出される関数。
 * ここで、cookieの計算を行い、script_(space_name)で読み込むjsファイルを特定する。
 * また、その後正しく書き出しが行われたかどうかを確認するために、checkBannerを予約する。
 * @param space_name
 */
function triggerBanner(space_name) {
	
	
	// 一定時間後に表示されたかチェックする。
	setTimeout("checkBanner('" + space_name + "')", check_error_msec);
	
	var html = '';
	
	var url = 'http://' + sannex_server_domain + '/scripts/sannex_banner_' + space_name + '.js';
	
	// リクエスト実施
	var script     = document.createElement( 'script' );
	script.type    = 'text/javascript';
	script.charset = 'utf-8';
//	script.charset = 'shift_jis';
	script.src     = url;
	document.getElementsByTagName("head")[0].appendChild( script );
	
	
	//alert('triggerBanner sannex_banner_' + space_name + '.js end.');
}


/**
 * 読み込んだ外部jsファイルからcall backされる関数。(JSONP)
 * space_(space_name)にバナーの内容を書き出す。
 * cookieのセットを行う。
 * 書き出し成功を記録する。
 * @param data
 */
function showBanner(data) {
	
	
	var space_name = data["space_name"];
	
	// エラーが既に出ていたら、もう置き換えない
	if ((check_banner_display[space_name] != undefined) && (check_banner_display[space_name] >= 1)) {
		debugOutput("エラーバナー表示済み:" + space_name + "<br />\n");
		return;
	} else {
		debugOutput("エラーバナー未表示:" + space_name + "<br />\n");
	}
	
	
	var banner_list = data["banner_list"];
	
	// 表示可能なモールだけ抽出
	mall_cookie_valid_days = data["cookie_valid_days"];
	var valid_mall_list = getValidMallList(mall_cookie_valid_days);
	
	// バナーを選ぶ。
	var banner = pickBanner(banner_list, valid_mall_list);
	
	// 表示する。
	showBannerHtml(space_name, banner);
	
	
	// 表示したことを記録する。
	check_banner_display[space_name] = 1;
}


/**
 * 実際に、指定されたspaceにbannerを表示します。
 */
function showBannerHtml(space_name, banner) {
	
	var target = document.getElementById('space_' + space_name);
	
	if (target != undefined) {
		target.innerHTML = banner.body;
	}
	debugOutput(space_name + "に表示したバナー:" + banner.mall_name + "<hr />\n");
	
	
	// スペース内のリンク先を置換して、クリックを記録できるようにする。
//	var link_list = target.getElementsByTagName('a');
	
//	for (var x = 0; x < link_list.length; x++) {
//		//debugOutput(link_list[x].innerHTML);
//		var tmp = link_list[x];
//		
//		//debugOutput(tmp.href);
//		
//		tmp_url = tmp.href;
//		tmp.href = "#";
//		tmp.onclick = function () {
//			setClickCookie(banner.mall_name, tmp_url);
//		}	
//	}
}

/**
/* クッキーをセット後、遷移。
 * 
 * @param mall_name モール名
 * @param url 遷移先
 */
function setClickCookie(mall_name, url) {
	//alert(mall_name);
	
	var dd = new Date();
	
	var valid_days = 1;
	if (mall_cookie_valid_days[mall_name] != undefined) {
		valid_days = mall_cookie_valid_days[mall_name];
	}
	
	writeCookie('BANNER_LAST_CLICK_' + mall_name, dd.getTime(), valid_days);
	
	document.location.href = url;
}



/**
 * 取得した、本体側のcookie有効設定から、いまユーザーに表示することが可能なモールの一覧を返す。
 * @access private
 * @param cookie_valid_days
 * @return Array モールの名前一覧
 */
function getValidMallList(cookie_valid_days) {
	debugOutput("利用可能なモールを選択します。<br />");
	
	var dd = new Date();
	
	var dtmp = new Date();
	
	// 最後に、各モールを表示したUTC時間を復元する。
	for (var x = 0; x < mall_name_list.length; x++) {
		var tmp = readCookie('BANNER_LAST_CLICK_' + mall_name_list[x]);
		
		dtmp.setTime(tmp);
		
		debugOutput("最終クリック時間:" + mall_name_list[x] + " " + tmp + " " + dtmp.toLocaleString() + "<br />");
		
		if ((tmp != undefined) && (tmp != "")) {
			banner_last_click[mall_name_list[x]] = tmp;
		} else {
			// 履歴がないものも0で初期化する。
			banner_last_click[mall_name_list[x]] = 0;
		}
	}
	
	var valid_mall_list = new Array();
	
	for (var x = 0; x < mall_name_list.length; x++) {
		
		// 有効期限がない=不明なモール
		if (cookie_valid_days[mall_name_list[x]] == undefined) {
			debugOutput("不明なモール:" + mall_name_list[x] + "<br />");
			continue;
		}
		
		// -1:ランダム 0:制限なし
		if (cookie_valid_days[mall_name_list[x]] <= 0) {
			valid_mall_list.push(mall_name_list[x]);
			continue;
		}
		
		// それ以外の場合は、最終表示時間から制限日数経過したもののみ有効
		var limit = (banner_last_click[mall_name_list[x]] * 1) + (cookie_valid_days[mall_name_list[x]] * 24 * 3600 * 1000);
		dtmp.setTime(limit);
		if (limit < dd.getTime()) {
			valid_mall_list.push(mall_name_list[x]);
			
			continue;
		}
		debugOutput("クッキー制限が有効時間内:" + mall_name_list[x] + ":" + cookie_valid_days[mall_name_list[x]] + " " + limit + " " + dtmp.toLocaleString() + " - " + dd.getTime() + " " + dd.toLocaleString() + "<br />\n");
		
	}
	
	return valid_mall_list;
}


/**
 * 表示するバナーを選びます。候補の中から、ウェイトで選ばれます。
 * @param banner_list
 * @param valid_mall_list
 * @return
 */
function pickBanner(banner_list, valid_mall_list) {
	//debugOutput("pickBanner start");
	
	var weight_total = 0;
	
	
	// 一時的に有効なバナーを入れる入れ物。
	var valid_banner_list = new Array();
	var first_valid_mall = null;
	
	for (var x = 0; x < banner_list.length; x++) {
		
		var tmp = banner_list[x];
		
		if (isInArray(tmp.mall_name, valid_mall_list)) {
			
			if (first_valid_mall == null) {
				// 1個目の有効なバナー
				valid_banner_list.push(tmp);
				weight_total += tmp.weight;
				first_valid_mall = tmp.mall_name;
			} else if (first_valid_mall == tmp.mall_name) {
				// 1個目の有効なバナーと同じモールを持つバナー
				valid_banner_list.push(tmp);
				weight_total += tmp.weight;
			}
			
		} else {
			//debugOutput("[Needle Not Found:" + tmp.mall_name + "]<br />\n")
		}
		
	}
	
	// 1個も有効なバナーがなかった場合は、ランダムに選ばせる。
	if (valid_banner_list.length < 1) {
		debugOutput("クッキー制限がかかっていないバナーが存在しません。<br />");
		return pickRandomBanner(banner_list);
	}
	
	
	var randnum = Math.floor( Math.random() * weight_total ); 
	
	// シャッフル
	valid_banner_list = valid_banner_list.shuffle();
	
	// randnumの値から、weightを減算していき、行き当たったらそれを表示。
	for (var x = 0; x < valid_banner_list.length; x++) {
		var tmp = valid_banner_list[x];
		if (weight_total - tmp.weight < 0) {
			// ウェイト値を下回ったバナーが、選ばれたバナー
			
			debugOutput("picked:" + tmp.mall_name);
			return tmp;
		}
	}
	
	// ランダムではなく、１個目を利用
	return valid_banner_list[0];
	
}


/**
 * バナー一覧から、ランダムにバナーを選びます。完全ランダム。
 * @return
 */
function pickRandomBanner(banner_list) {
	debugOutput("ランダムにバナーを選択します。<br />\n");
	
	if (banner_list.length < 1) {
		return null;
	}

	
	var weight_total = 0;
	
	
	// 一時的に有効なバナーを入れる入れ物。
	var valid_banner_list = new Array();
	
	for (var x = 0; x < banner_list.length; x++) {
		
		var tmp = banner_list[x];
		weight_total += (tmp.weight * 1);
		
	}
	var randnum = Math.floor( Math.random() * weight_total ); 
	//debugOutput("randnum1:" + weight_total + " " + randnum + "<br />\n");
	
	// シャッフル
	banner_list = banner_list.shuffle();
	
	// randnumの値から、weightを減算していき、行き当たったらそれを表示。
	for (var x = 0; x < banner_list.length; x++) {
		var tmp = banner_list[x];
		if (randnum - tmp.weight < 0) {
			// ウェイト値を下回ったバナーが、選ばれたバナー
			
			debugOutput("ランダムに選択:" + tmp.mall_name + " " + randnum + " " + tmp.weight + "<br />\n");
			return tmp;
		}
		randnum -= tmp.weight;
	}
	
	//debugOutput("randnum2:" + randnum + "<br />\n");
	//var randnum = Math.floor( Math.random() * banner_list.length ); 
	
	//var banner = banner_list[randnum];
	var banner = banner_list[0];
	
	return banner;
}

/**
 * showBannerが正しく呼び出されたかを確認する関数。
 * triggerBannerで予約される。showBannerが実行されていなければ、showErrorbannerを呼び出す。
 * @param space_name
 */
function checkBanner(space_name) {
	
	if ((check_banner_display[space_name] != undefined) && (check_banner_display[space_name] == 1)) {
		debugOutput("バナーの読み込みチェック OK:" + space_name + "<br />\n");
	} else {
		
		debugOutput("バナーの読み込みチェック NG:" + space_name + "<br />\n");
		
		// エラー用のバナー一覧を取得する
		var banner_list = getErrorBanner(space_name);
		//var banner = pickRandomBanner(banner_list);
		
		// エラー時も、クッキー制御が有効になるようにする
		// エラー用データからクッキー有効期限
		mall_cookie_valid_days = getErrorCookieSettings();
		var valid_mall_list = getValidMallList(mall_cookie_valid_days);
		// バナーを選ぶ。
		var banner = pickBanner(banner_list, valid_mall_list);
		
		
		
		// 表示する
		showBannerHtml(space_name, banner);
		
		// エラーが表示されたことを記録
		check_banner_display[space_name] = 2;
	}
	
}

/**
 * Cookie への書き出し
 * 引数 key  : データキー （半角英数 _ のみ）
 * 引数 value : データの値（日本語可）
 * 引数 days  : データを保持する日数(0 の時は有効期限は省略)
 */
function writeCookie(key, value, days) {
	var str = key + "=" + escape(value) + ";";         // 書き出す値１ : key=value
	if (days != 0) {                                                 /* 日数 0 の時は省略 */
		var dt = new Date();                                   // 現在の日時
		dt.setDate(dt.getDate() + days);                   // days日後の日時
		str += "expires=" + dt.toGMTString() + ";"; // 書き出す値２ : 有効期限
	}
	str += "path=/;";
	document.cookie = str;                                   // Cookie に書き出し
}

/**
 * Cookie の読み込み
 * 引数 key : 求める値のキー
 * 戻り値 : 値（ない時は空文字""）
 */
function readCookie(key) {
	var sCookie = document.cookie;    // Cookie文字列
	var aData = sCookie.split(";");	  // ";"で区切って"キー=値"の配列にする
	var oExp = new RegExp(" ", "g");   // すべての半角スペースを表す正規表現
	key = key.replace(oExp, "");		// 引数keyから半角スペースを除去

	var i = 0;
	while (aData[i]) {					  /* 語句ごとの処理 : マッチする要素を探す */
		var aWord = aData[i].split("=");					// さらに"="で区切る
		aWord[0] = aWord[0].replace(oExp, "");		    // 半角スペース除去
		if (key == aWord[0]) {
			// マッチしたら値を返す
			return unescape(aWord[1]);
		}
		if (++i >= aData.length) {
			// 要素数を超えたら抜ける
			break;
		}
	}
	// 見つからない時は空文字を返す
	return "";
}


/**
 * 配列の中に該当の値があるか？jQueryと競合しないようにisInArrayに変更。
 * @param needle
 * @param haystack
 * @return
 */
function isInArray( needle, haystack ){
	var found = false;
	for(var i = 0; i < haystack.length; i++){
		if(needle == haystack[i]){
			found = true;
			break;
		}
	}
	return found;
}


/**
 * テスト用にクッキーを削除します。
 * @return
 */
function resetCookie() {
	
	for (var x = 0; x < mall_name_list.length; x++) {
		writeCookie('BANNER_LAST_CLICK_' + mall_name_list[x], 0);
	}
	debugOutput("全モールのクッキーを初期化しました。<br />\n");
}

/**
 * デバッグ出力します。
 * @return
 */
function debugOutput(body) {
	
	var debug = document.getElementById('debug');
	
	if (debug != undefined) {
		debug.innerHTML = debug.innerHTML + body;
	}
}

// 初期化実行。
init();

