﻿////////////////////////////////////////////////////////////////////////////////module deqwas2
//// Version 2.0.4
function DeqwasAgent() {
    this.initialize.apply(this, arguments);
}

DeqwasAgent.prototype = {
    codeVersion: 'xxx',
    collectionType: { normal: 'i', cart: 'c', shipping: 's' },
    collectionElementId: 'deqwas-collection',
    screenType: { normal: 'r', cart: 'k', shipping: 'h' },
    screenElementId: 'deqwas-screen',
    visibleFunctionIds: { r: true, k: true, h: true },

    initialize: function (site, objectType, deqwasObject, scriptType) {

        this.site = site;
        this.objectType = objectType;
        this.objectId = deqwasObject.id;
        this.scriptType = scriptType;

        this.parameters = {
            l: deqwasObject.location ? deqwasObject.location : location.href,
            fc: "",
            cid: this.site,
            iid: deqwasObject.id,
            uid: deqwasObject.viewer_id,
            title: deqwasObject.name,
            img: deqwasObject.image,
            val: deqwasObject.value,
            category: deqwasObject.category,
            caption: deqwasObject.caption,
            info: deqwasObject.info,
            role: '',
            logic: '',
            place: '',
            essential: ''
        };

        this.role = '';
        this.logic = '';
        this.place = '';
        this.essential = '';
        this.exclusion = deqwasObject.option.exclusion;

        if (deqwasObject.option.silence) {
            this.essential = 'confirmation';
        }
        this.noscreen = deqwasObject.option.noscreen;
        if (deqwasObject.option.noscreen || deqwasObject.option.solitude) {
            this.expression = deqwasObject.option.noscreen ? 'F' : 'X';
            this.expression += deqwasObject.option.solitude ? 'T' : 'X';
        }

        this.targetUrl = '';
        this.extension = '';
        this.baseXmlUrl = '';

        this.xslName = '';
        this.xslDirectory = '';

        this.candidates = [];

        this.iframeStyle = {
            width: '0px',
            height: '0px'
        };
        this.iframeOptions = {
            frameBorder: "0",
            scrolling: "no"
        };
    },

    getSafeProperty: function (property) {
        return Object.prototype.hasOwnProperty.call(window, property)
      ? window[property] : "";
    },

    _setCookie: function (key, val, path, expires) {
        var cookie = [key + '=' + escape(val),
                   'path' + '=' + path,
                   'expires' + '=' + expires
                 ].join(';');
        document.cookie = cookie;
    },

    _getCookie: function (key) {
        var cookieKey = key + "=";
        var val = null;
        var cookie = document.cookie + ";";
        var index = cookie.indexOf(cookieKey);
        if (index != -1) {
            var endIndex = cookie.indexOf(";", index);
            val = unescape(cookie.substring(index + cookieKey.length, endIndex));
        }
        return val;
    },

    deleteCookie: function (key, path) {
        var commingTime = new Date();
        commingTime.setFullYear(commingTime.getFullYear() - 1);
        var expires = commingTime.toGMTString() + ";";
        this._setCookie(key, '', path, expires);
    },

    _createRandomString: function () {
        var string = "";
        var charArray = ['1', '2', '3', '4', '5', '6', '7', '8'];
        for (var i = 0; i < 18; i++) {
            string += charArray[Math.floor(Math.random() * charArray.length)];
        }
        return string;
    },

    setViewerCookie: function () {
        var origin = "ra000000000000" + this.codeVersion + this.site;


        // add 20100903
        visit = "1";
        if (this._getCookie(origin) === null) {
            visit = "0";
        }

        // add 20100903
        this.parameters["visit"] = visit;

        var cookieValue = this._getCookie(origin) || this._createRandomString();

        var time = new Date();
        time.setTime(time.getTime() + (1000 * 60 * 60 * 24 * 365));
        var expires = time.toGMTString();

        this._setCookie(origin, cookieValue, '/', expires);
        this.parameters[origin] = cookieValue;

    },

    _createIframeSrc: function (parameters, targetUrl) {
        var parameterArray = [];
        // clean Object.prototype
        for (var parameter in new Object()) {
            parameters[parameter] = undefined;
        }

        for (var key in parameters) {
            if (parameters[key] && typeof parameters[key] != 'function') {
                parameters[key] = (typeof parameters[key] == 'string') ? parameters[key].replace(/[\f\n\r\t\v]/g, '') : parameters[key];
                parameterArray.push(key + "=" + encodeURIComponent(parameters[key]));
            }
        }
        var iframeSrc = targetUrl + "?" + parameterArray.join("&");
        return iframeSrc;
    },

    _createSourceParameter: function (baseXmlUrl, site, objectType, objectId, extension) {
        return [baseXmlUrl, site, objectType, objectId + extension].join('/');
    },

    setTargetUrl: function (targetUrl) {
        this.targetUrl = targetUrl;
    },

    // deprecated
    setExtraParams: function (extraParams) {
        this.setExtraParameters(extraParams);
    },

    setExtraParameters: function (extraParameters) {
        for (var key in extraParameters) {
            this.parameters[key] = extraParameters[key];
        }
    },

    // deprecated
    setSourceParam: function (extension, baseXmlUrl) {
        this.setSource(extension, baseXmlUrl);
    },

    // deprecated
    setSource: function (extension, baseXmlUrl) {
        if (extension) this.setExtension(extension);
        if (baseXmlUrl) this.setBaseXmlUrl(baseXmlUrl);
    },

    setRole: function (role) {
        this.role = role;
    },

    setLogic: function (logic) {
        this.logic = logic;
    },

    setPlace: function (place) {
        this.place = place;
    },

    setEssential: function (essential) {
        this.essential = essential;
    },

    setNextCandidateId: function (objectId) {
        this.candidates.push(this._createSourceParameter(
                      this.baseXmlUrl
                      , this.site
                      , this.objectType
                      , objectId
                      , this.extension));
    },

    setNextCandidateParameter: function (parameter) {
        this.candidates.push(parameter);
    },

    setExtension: function (extension) {
        this.extension = extension;
    },

    setBaseXmlUrl: function (baseXmlUrl) {
        this.baseXmlUrl = baseXmlUrl;
    },

    // deprecated
    setTypeParam: function (xslName, xslDirectory) {
        this.setType(xslName, xslDirectory);
    },

    // deprecated
    setType: function (xslName, xslDirectory) {
        if (xslName) this.setXslName(xslName);
        if (xslDirectory) this.setXslDirectory(xslDirectory);
    },

    setXslName: function (xslName) {
        this.xslName = xslName;
    },

    setXslDirectory: function (xslDirectory) {
        this.xslDirectory = xslDirectory;
    },

    setScreenName: function (screenName) {
        this.screenElementId = [this.screenElementId, screenName].join('-');
    },

    setCollectionElementId: function (elementId) {
        this.collectionElementId = elementId;
    },

    setScreenElementId: function (elementId) {
        this.screenElementId = elementId;
    },

    setIframeStyle: function (width, height, styleObject) {
        if (!width || !height) return false;
        styleObject = styleObject || {};

        this.iframeStyle.width = width;
        this.iframeStyle.height = height;
        for (var key in styleObject) {
            this.iframeStyle[key] = styleObject[key];
        }
    },

    setIframeOptions: function (iframeOptions) {
        if (!iframeOptions) return false;
        for (var key in iframeOptions) {
            this.iframeOptions[key] = iframeOptions[key];
        }
    },

    appendIframeToElement: function (functionId) {
        var elementId;
        if (!functionId) {
            if (document.getElementById(this.screenElementId)) {
                functionId = this.screenType[this.scriptType];
            }
            else if (document.getElementById(this.collectionElementId)) {
                functionId = this.collectionType[this.scriptType];
            }
            else {
                return false;
            }
        }

        if (this.noscreen) {
            functionId = this.collectionType[this.scriptType];
        }

        if (this.exclusion) {
            functionId = 'x';
            elementId = this.collectionElementId;
            this.iframeStyle = {
                width: '0px',
                height: '0px'
            };
        }
        else if (this.visibleFunctionIds[functionId]) {
            elementId = this.screenElementId;
            this.parameters.source = this._createSourceParameter(
                                 this.baseXmlUrl
                                 , this.site
                                 , this.objectType
                                 , this.objectId
                                 , this.extension);
            for (var i = 0; i < this.candidates.length; i++) {
                this.parameters['source' + (i + 2)] = this.candidates[i];
            }
            this.parameters.type = [this.xslDirectory
                              , this.site
                              , this.xslName].join('/');
        }
        else {
            elementId = this.collectionElementId;
            this.iframeStyle = {
                width: '0px',
                height: '0px'
            };
        }
        if (!document.getElementById(elementId)) {
            return false;
        }

        this.parameters.fc = functionId;
        if (this.essential) {
            this.parameters.essential = this.essential;
        }
        this.parameters.role = this.role;
        this.parameters.logic = this.logic;
        this.parameters.place = this.place;
        this.parameters.essential = this.essential;
        this.parameters.expression = this.expression;

        //this.checkDebug();

        var iframeName = "deqwas_" + functionId;
        var iframe = document.createElement('iframe');

        iframe.src = this._createIframeSrc(this.parameters, this.targetUrl);
        var iframeStyleString = "";
        for (var key in this.iframeStyle) {
            iframeStyleString += [key, ':', this.iframeStyle[key], ';'].join(' ');
        }
        iframe.style.cssText = iframeStyleString;
        for (var attr in this.iframeOptions) {
            iframe[attr] = this.iframeOptions[attr];
        }
        iframe.name = iframeName;

        //TODO check exist iframe
        document.getElementById(elementId).appendChild(iframe);

    },

    // リファラをセット
    setReferrer: function () {
        this.parameters["ref"] = document.referrer;
    },

    // ランダム値をセット
    setRandom: function (val) {
        this.parameters["ran"] = val;
    },

    clearParam: function () {
        this.parameters["l"] = null;
        this.parameters["fc"] = null;
        this.parameters["title"] = null;
        this.parameters["essential"] = null;
        this.parameters["visit"] = null;
        this.parameters["ra000000000000" + deqwasCid] = null;
    },

    // ユーザを一意に紐づけるランダム値
    createRandomNumber: function () {
        var value = "1";
        var characters = '0123456789';
        for (var i = 0; i < 15; i++) {
            value += characters.charAt(Math.floor(Math.random() * characters.length));
        }
        return value;
    }
};
//////////////////////////////////////////////////////////////////////////////// 初期処理
// 画面名
deqwas.name = document.title;

var deqwasCid;

// 顧客サイト名
deqwasCid = deqwas.cid;
if (deqwasCid == null) {
    deqwasCid = "adresult"
}

//////////////////////////////////////////////////////////////////////////////// ユーザ情報取得
var deqwasAgent = new DeqwasAgent(deqwasCid, "item", deqwas, "normal");
var deqwasRandom = deqwasAgent.createRandomNumber();

// ランダム値の設定
deqwasAgent.setRandom(deqwasRandom);
// 念のためセット ※未使用
deqwasAgent.setViewerCookie();
// 必要のないパラメータをクリア
deqwasAgent.clearParam();

// iframeのsrc属性
//deqwasAgent.targetUrl = "/adresult/Collection.aspx"; // Debug用
deqwasAgent.targetUrl = location.protocol + "//dsp07.deqwas.net/adresult/Collection.aspx"; // Release用

// iframeのstyle属性
deqwasAgent.iframeStyle = { width: '0px', height: '0px' };

// iframeの属性
//deqwasAgent.iframeOptions = { frameBorder: "0", scrolling: "no" };

// キーワード設定
var deqwasKeywordsLimit = 50;

var deqwasKeywords = (function () {
    var metas = document.getElementsByTagName("meta");

    for (var i = 0; i < metas.length; i++) {
        var meta = metas[i];
        var name = meta.name;

        if (name.match(/keywords/i)) {
            return meta.content;
        }
    }
})();

if (deqwasKeywords) {
    if (deqwasKeywords.length > deqwasKeywordsLimit) {
        deqwasKeywords = deqwasKeywords.substr(0, deqwasKeywords.substr(0, deqwasKeywordsLimit).lastIndexOf(","));
    }
}

deqwasAgent.setExtraParameters({
    kw: deqwasKeywords
});

// インラインフレームの追加
deqwasAgent.appendIframeToElement();

//////////////////////////////////////////////////////////////////////////////// リファラー取得
var deqwasAgentRef = new DeqwasAgent(deqwasCid, "item", deqwas, "normal");

// ランダム値の設定
deqwasAgentRef.setRandom(deqwasRandom);

// 必要のないパラメータをクリア
deqwasAgentRef.clearParam();

// iframeのsrc属性
//deqwasAgentRef.targetUrl = "/adresult/Referrer.aspx"; // Debug用
deqwasAgentRef.targetUrl = location.protocol + "//dsp07.deqwas.net/adresult/Referrer.aspx"; // Release用

// リファラの設定
deqwasAgentRef.setReferrer();

// インラインフレームの追加
deqwasAgentRef.appendIframeToElement("ref");


//////////////////////////////////////////////////////////////////////////////// URL取得
var deqwasAgentUrl = new DeqwasAgent(deqwasCid, "item", deqwas, "normal");

// ランダム値の設定
deqwasAgentUrl.setRandom(deqwasRandom);

// 必要のないパラメータをクリア
deqwasAgentUrl.clearParam();

// URLを設定
deqwasAgentUrl.parameters["l"] = deqwas.location ? deqwas.location : location.href;

// iframeのsrc属性
//deqwasAgentUrl.targetUrl = "/adresult/Url.aspx"; // Debug用
deqwasAgentUrl.targetUrl = location.protocol + "//dsp07.deqwas.net/adresult/Url.aspx"; // Release用

// インラインフレームの追加
deqwasAgentUrl.appendIframeToElement("url");
